package cn.idaddy.open.demo.service;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import cn.idaddy.open.demo.common.Strings;
import cn.idaddy.open.demo.common.exceptions.BizError;
import cn.idaddy.open.demo.common.exceptions.ServiceFailure;
import cn.idaddy.open.demo.service.dto.AudioDto;
import cn.idaddy.open.demo.service.dto.DeviceTokenDto;
import cn.idaddy.open.demo.service.dto.ResponseDto;
import okhttp3.HttpUrl;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

public class OpenService {
	private ObjectMapper mapper = new ObjectMapper();
	{
		mapper.registerModule(new JavaTimeModule());
	}
	private OkHttpClient client = new OkHttpClient();

	private static HttpUrl url(String pathSegments) {
		return new HttpUrl.Builder().scheme("http").host("open.idaddy.cn").addPathSegments(pathSegments).build();
	}

	public static String addDeviceToken(String url,String device_token){
		if(Strings.isNullOrEmpty(device_token)){
			throw new RuntimeException("required device_token");
		}
		if(Strings.isNullOrEmpty(url)){
			return url;
		}
		return HttpUrl.parse(url).newBuilder()
				.addQueryParameter("token",device_token)
				.build()
				.toString();
	}

	public static void main(String[] args) {
        System.out.println(url("audio/v2/refresh_device_token"));
        System.out.println(PostMap.encodeUrl(1L));
        System.out.println(new OpenService().json2object("{}",
				new TypeReference<ResponseDto<DeviceTokenDto>>() {}));
	}

	/**
	 * {
	 *  "data": {
	 *     "device_token": "TJVA95OrM7E2cBab30RMHrHDcEfxjoYZgeFON",
	 *     "device_id": "x1000",
	 *     "created_at": "2017-06-18T00:07:08Z",
	 *     "expires_at": "2017-06-23T00:07:08Z"
	 *   },
	 *  "retcode": 0
	 * }
	 */
	public DeviceTokenDto refresh_device_token(AppSign sign, String device_id) {
        PostMap map=PostMap.from(sign);
        map.put("device_id",device_id);
		Request request = new Request.Builder()
				.url(url("audio/v2/refresh_device_token"))
				.post(RequestBody.create(MediaType.parse("application/x-www-form-urlencoded; charset=UTF-8"),map.toString()))
				.build();
		String bodyString=request(request);
		ResponseDto<DeviceTokenDto> body = json2object(bodyString, new TypeReference<ResponseDto<DeviceTokenDto>>() {});

		if (!body.isOk()) {
			throw new BizError("请求失败：" + body.msg);
		}
		return body.data;
	}

	/**
	 curl --request POST \
	 --data-urlencode "app_id=${APP_ID}" \
	 --data-urlencode "device_id=x1000" \
	 --data-urlencode "timestamp=${TIMESTAMP}" \
	 --data-urlencode "nonce=${NONCE}" \
	 --data-urlencode "signature=${SIGNATURE}" \
	 --data-urlencode "offset=0" \
	 --data-urlencode "limit=2" \
	 --data-urlencode 'content=世上只有妈妈好' \
	 --data-urlencode "verbose=0" \
	 http://open.idaddy.cn/audio/v2/query

	 {
		 "audioinfos": {
			 "cats": [],
			 "contents": [
				 {
					 "chapters": [
						 {
							 "status": 0,
							 "name": "翡翠村的熊猫",
							 "lyrics": "",
							 "play_url": "http://cdn.open.idaddy.cn/apsmp3/4e09/idaddy0000000006/201706150000/1/YTk2LzkvYWd1cndjZ3ouYXVkaW8=.mp3",
							 "chapter_id": "ADQGNlA1DTM=",
							 "free": 1,
							 "duration": 452
						 }
					 ],
					 "duration": 0,
					 "has_chapter": 1,
					 "author": "zpe9",
					 "play_url": "",
					 "id": "ADYGPlA8DTw=",
					 "audio_name": "翡翠村的熊猫",
					 "chapter_count": 21,
					 "name": "翡翠村的熊猫",
					 "is_multichapter": 1,
					 "icon": "http://img.idaddy.cn//b/9/n60s6oyo.jpg"
				 }
			 ]
		 },
		 "retcode": 0
	 }
	 */
	public List<AudioDto> find_audio(AppSign sign, String device_id, String keyword, int offset, int limit) {
		PostMap map=PostMap.from(sign);
		map.put("device_id",device_id);
		map.put("content",keyword);
		map.put("offset",offset);
		map.put("limit",limit);
		Request request = new Request.Builder()
				.url(url("audio/v2/query"))
				.post(RequestBody.create(MediaType.parse("application/x-www-form-urlencoded; charset=UTF-8"),map.toString()))
				.build();
		String bodyString=request(request);
		ResponseDto<Void> body = json2object(bodyString, new TypeReference<ResponseDto<Void>>() {});

		if (!body.isOk()) {
			throw new BizError("请求失败：" + body.msg);
		}
		if(body.audioinfos==null) return Collections.emptyList();
		if(body.audioinfos.contents==null) return Collections.emptyList();
		return body.audioinfos.contents;
	}


	private String request(Request request) {
		Response response;
		try {
			response = client.newCall(request).execute();
		} catch (IOException e) {
			throw new ServiceFailure("请求失败:audio/v2/refresh_device_token ", e);
		}
		try {
			return response.body().string();
		} catch (IOException e) {
			throw new ServiceFailure("读取响应失败:audio/v2/refresh_device_token ", e);
		}
	}

	private <T> T json2object(String json, TypeReference<T> valueTypeRef) {
		try {
			return mapper.readValue(json, valueTypeRef);
		} catch (IOException e) {
			throw new ServiceFailure("Failed to json2object JSON ", e);
		}
	}
    private static class PostMap extends LinkedHashMap<String,Object>{

        /**
		 * 
		 */
		private static final long serialVersionUID = -5104605247336662930L;

		public PostMap() {
        }

        //    curl --request POST \
//            --data-urlencode "app_id=${APP_ID}" \
//            --data-urlencode "timestamp=${TIMESTAMP}" \
//            --data-urlencode "nonce=${NONCE}" \
//            --data-urlencode "signature=${SIGNATURE}" \
//            --data-urlencode "device_id=x1000" \
//    http://open.idaddy.cn/audio/v2/refresh_device_token
        private static PostMap from(AppSign sign) {
            PostMap map=new PostMap();
            map.put("app_id",sign.getApp_id());
            map.put("timestamp",sign.getTimestamp());
            map.put("nonce",sign.getNonce());
            map.put("signature",sign.getSignature());
            return map;
        }
        /**
         * map to application/x-www-form-urlencoded
         */
        private static String toFormUrl(Map<String, Object> map) {
            StringBuilder query=new StringBuilder();
            Iterator<String> keys = map.keySet().iterator();
            while (keys.hasNext()) {
                String key = keys.next();
                query.append(key+"="+ encodeUrl(map.get(key)));
                if(keys.hasNext()) {
                    query.append("&");
                }
            }
            return query.toString();
        }

        @Override
        public String toString() {
            return toFormUrl(this);
        }

        private static String encodeUrl(Object value) {
            if(value==null)return "";
            try {
                return URLEncoder.encode(value.toString(),"UTF-8");
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException("bug: encoding is not supported:UTF-8");
            }
        }
    }




}